﻿
Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function

    
    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function



    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            'f.WriteLine(TabString("Form1_Width", r.Width))
            'f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine("")
            f.WriteLine(" InOut")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SlotIn", Form1.txt_SlotIn.Text))
            f.WriteLine(TabString("SlotInChange", Form1.txt_SlotInChange.Text))
            f.WriteLine(TabString("SlotOut", Form1.txt_SlotOut.Text))
            f.WriteLine(TabString("SlotOutChange", Form1.txt_SlotOutChange.Text))
            f.WriteLine(TabString("SlotPan", Form1.txt_SlotPan.Text))
            f.WriteLine(TabString("SlotVolUp", Form1.txt_SlotVolUp.Text))
            f.WriteLine(TabString("SlotVolDw", Form1.txt_SlotVolDw.Text))
            f.WriteLine(TabString("SlotFilterFreq", Form1.txt_SlotFilterFreq.Text))
            f.WriteLine(TabString("SlotFilterRes", Form1.txt_SlotFilterRes.Text))
            f.WriteLine(TabString("SlotEchoVolume", Form1.txt_SlotEchoVolume.Text))
            f.WriteLine(TabString("SlotEchoFbk", Form1.txt_SlotEchoFbk.Text))
            f.WriteLine("")
            f.WriteLine(" Position")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Position", Form1.tk_Position.Value.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Audio file")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("WaveFileName", Form1.cmb_WaveFiles.Text))
            f.WriteLine(TabString("Change", Form1.chk_Change.Checked.ToString))
            f.WriteLine(TabString("Random", Form1.chk_Random.Checked.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Trigger-Mode TabIndex")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("TriggerMode", Form1.TabControl1.SelectedIndex.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Scratch Controls")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ResponseSpeed", Form1.txt_ResponseSpeed.NumericValueInteger.ToString))
            f.WriteLine(TabString("Margin", Form1.txt_Margin.NumericValueInteger.ToString))
            f.WriteLine(TabString("MaxSpeedUP", Form1.txt_MaxSpeedUP.NumericValue.ToString))
            f.WriteLine(TabString("MaxSpeedDW", Form1.txt_MaxSpeedDW.NumericValue.ToString))
            f.WriteLine(TabString("Follow", Form1.chk_Follow.Checked.ToString))
            f.WriteLine(TabString("Looping", Form1.chk_Looping.Checked.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Trig Controls")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("TrigSensitivity", Form1.txt_TrigSensitivity.NumericValueInteger.ToString))
            f.WriteLine(TabString("PlaySpeed", Form1.txt_PlaySpeed.NumericValue.ToString))
            f.WriteLine(TabString("PlaySpeedRandom", Form1.txt_PlaySpeedRandom.NumericValue.ToString))
            f.WriteLine(TabString("PlaySpeedMin", Form1.txt_PlaySpeedMin.NumericValue.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" OutTab TabIndex")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("OutTabIndex", Form1.TabControl2.SelectedIndex.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Audio out")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("OutDevice", Form1.txt_OutDevice.Text))
            f.WriteLine(TabString("Playing", Form1.chk_Play.Checked.ToString))
            f.WriteLine(TabString("Stereo", Form1.chk_Stereo.Checked.ToString))
            f.WriteLine(TabString("Pan", Form1.tk_Pan.Value.ToString))
            f.WriteLine(TabString("VolumeUP", Form1.tk_VolumeUP.Value.ToString))
            f.WriteLine(TabString("VolumeDW", Form1.tk_VolumeDW.Value.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Filter")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("FilterType", Form1.cmb_FilterType.Text))
            f.WriteLine(TabString("FilterEnabled", Form1.chk_FilterEnabled.Checked.ToString))
            f.WriteLine(TabString("FilterFreq", Form1.tk_FilterFreq.Value.ToString))
            f.WriteLine(TabString("FilterRes", Form1.tk_FilterRes.Value.ToString))
            f.WriteLine(TabString("FilterDrive", Form1.tk_FilterDrive.Value.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Echo")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("EchoEnabled", Form1.chk_EchoEnabled.Checked.ToString))
            f.WriteLine(TabString("EchoVolume", Form1.tk_EchoVolume.Value.ToString))
            f.WriteLine(TabString("EchoFbk", Form1.tk_EchoFbk.Value.ToString))
            f.WriteLine(TabString("EchoDelay", Form1.tk_EchoDelay.Value.ToString))
            '
            'f.WriteLine("")
            'f.WriteLine(" examples")
            'f.WriteLine("===========================================")
            'f.WriteLine(TabString("DefaultCapMaxDist", Form1.txt_MaxDist.NumericValue))
            'f.WriteLine(TabString("FastScroll", Form2.btn_FastScroll.Checked))
            'f.WriteLine(TabString("ShowRawCount", Form2.btn_ShowRawCount.Checked))
            'f.WriteLine(TabString("ShowXY", Form2.btn_ShowXY.Checked))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI()
        ' ------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : Form1.Left = CInt(Val(l))
                        'Case "Form1_Width" : Form1.Width = CInt(Val(l))
                        'Case "Form1_Height" : Form1.Height = CInt(Val(l))
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' ----------------------------------------------------------------- InOut
                    Case "SlotIn" : Form1.txt_SlotIn.Text = l
                    Case "SlotInChange" : Form1.txt_SlotInChange.Text = l
                    Case "SlotOut" : Form1.txt_SlotOut.Text = l
                    Case "SlotOutChange" : Form1.txt_SlotOutChange.Text = l
                    Case "SlotPan" : Form1.txt_SlotPan.Text = l
                    Case "SlotVolUp" : Form1.txt_SlotVolUp.Text = l
                    Case "SlotVolDw" : Form1.txt_SlotVolDw.Text = l
                    Case "SlotFilterFreq" : Form1.txt_SlotFilterFreq.Text = l
                    Case "SlotFilterRes" : Form1.txt_SlotFilterRes.Text = l
                    Case "SlotEchoVolume" : Form1.txt_SlotEchoVolume.Text = l
                    Case "SlotEchoFbk" : Form1.txt_SlotEchoFbk.Text = l
                        ' ----------------------------------------------------------------- Position
                    Case "Position" : Form1.tk_Position.Value = CInt(Val(l))
                        ' ----------------------------------------------------------------- Audio file
                    Case "WaveFileName" : Combo_SetIndex_FromString(Form1.cmb_WaveFiles, l)
                    Case "Change" : Form1.chk_Change.Checked = l = "True"
                    Case "Random" : Form1.chk_Random.Checked = l = "True"
                        ' ----------------------------------------------------------------- Trigger-Mode TabIndex
                    Case "TriggerMode" : Form1.TabControl1.SelectedIndex = Val_Int(l)
                        ' ----------------------------------------------------------------- Scratch Controls
                    Case "ResponseSpeed" : Form1.txt_ResponseSpeed.NumericValueInteger = Val_Int(l)
                    Case "Margin" : Form1.txt_Margin.NumericValueInteger = Val_Int(l)
                    Case "MaxSpeedUP" : Form1.txt_MaxSpeedUP.NumericValue = Val_Double(l)
                    Case "MaxSpeedDW" : Form1.txt_MaxSpeedDW.NumericValue = Val_Double(l)
                    Case "Follow" : Form1.chk_Follow.Checked = l = "True"
                    Case "Looping" : Form1.chk_Looping.Checked = l = "True"
                        ' ----------------------------------------------------------------- Trig Controls
                    Case "TrigSensitivity" : Form1.txt_TrigSensitivity.NumericValueInteger = Val_Int(l)
                    Case "PlaySpeed" : Form1.txt_PlaySpeed.NumericValue = Val_Double(l)
                    Case "PlaySpeedRandom" : Form1.txt_PlaySpeedRandom.NumericValue = Val_Double(l)
                    Case "PlaySpeedMin" : Form1.txt_PlaySpeedMin.NumericValue = Val_Double(l)
                        ' ----------------------------------------------------------------- OutTab TabIndex
                    Case "OutTabIndex" : Form1.TabControl2.SelectedIndex = Val_Int(l)
                        ' ----------------------------------------------------------------- Audio out
                    Case "OutDevice" : Form1.txt_OutDevice.Text = l
                    Case "Playing" : Form1.chk_Play.Checked = l = "True"
                    Case "Stereo" : Form1.chk_Stereo.Checked = l = "True"
                    Case "Pan" : Form1.tk_Pan.Value = Val_Int(l)
                    Case "VolumeUP" : Form1.tk_VolumeUP.Value = Val_Int(l)
                    Case "VolumeDW" : Form1.tk_VolumeDW.Value = Val_Int(l)
                        ' ----------------------------------------------------------------- Filter
                    Case "FilterType" : Combo_SetIndex_FromString(Form1.cmb_FilterType, l)
                    Case "FilterEnabled" : Form1.chk_FilterEnabled.Checked = l = "True"
                    Case "FilterFreq" : Form1.tk_FilterFreq.Value = Val_Int(l)
                    Case "FilterRes" : Form1.tk_FilterRes.Value = Val_Int(l)
                    Case "FilterDrive" : Form1.tk_FilterDrive.Value = Val_Int(l)
                        ' ----------------------------------------------------------------- Echo
                    Case "EchoEnabled" : Form1.chk_EchoEnabled.Checked = l = "True"
                    Case "EchoVolume" : Form1.tk_EchoVolume.Value = Val_Int(l)
                    Case "EchoFbk" : Form1.tk_EchoFbk.Value = Val_Int(l)
                    Case "EchoDelay" : Form1.tk_EchoDelay.Value = Val_Int(l)
                        ' ----------------------------------------------------------------- examples
                        'Case "DefaultCapMaxDist" : Form1.txt_MaxDist.NumericValue = Val(l)
                        'Case "FastScroll" : Form2.btn_FastScroll.Checked = l = "True"
                        'Case "ShowRawCount" : Form2.btn_ShowRawCount.Checked = l = "True"
                        'Case "ShowXY" : Form2.btn_ShowXY.Checked = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub


End Module
