
Imports System.IO

' ========================================================================================================
'   CLASS - WAVE STREAM
' ========================================================================================================

Public Class WaveStream

    Inherits Stream
    Implements IDisposable
    Private m_Stream As Stream
    Private m_DataPos As Long
    Private m_Length As Long

    Private m_Format As WaveNative.WAVEFORMATEX

    Friend ReadOnly Property Format() As WaveNative.WAVEFORMATEX
        Get
            Return m_Format
        End Get
    End Property

    Private Function ReadChunk(ByVal reader As BinaryReader) As String
        Dim ch As Byte() = New Byte(3) {}
        reader.Read(ch, 0, ch.Length)
        Return System.Text.Encoding.ASCII.GetString(ch)
    End Function

    Private Sub ReadHeader()
        Dim Reader As BinaryReader = New BinaryReader(m_Stream)
        If ReadChunk(Reader) <> "RIFF" Then
            Throw New Exception("Invalid file format")
        End If

        Reader.ReadInt32()
        ' File length minus first 8 bytes of RIFF description, we don't use it
        If ReadChunk(Reader) <> "WAVE" Then
            Throw New Exception("Invalid file format")
        End If

        If ReadChunk(Reader) <> "fmt " Then
            Throw New Exception("Invalid file format")
        End If

        Dim len As Integer = Reader.ReadInt32()
        If len < 16 Then
            ' bad format chunk length
            Throw New Exception("Invalid file format")
        End If

        m_Format = New WaveNative.WAVEFORMATEX(22050, 16, 2)
        ' initialize to any format
        m_Format.wFormatTag = Reader.ReadInt16()
        m_Format.nChannels = Reader.ReadInt16()
        m_Format.nSamplesPerSec = Reader.ReadInt32()
        m_Format.nAvgBytesPerSec = Reader.ReadInt32()
        m_Format.nBlockAlign = Reader.ReadInt16()
        m_Format.wBitsPerSample = Reader.ReadInt16()

        ' advance in the stream to skip the wave format block 
        len -= 16
        ' minimum format size
        While len > 0
            Reader.ReadByte()
            len -= 1
        End While

        ' assume the data chunk is aligned
        While m_Stream.Position < m_Stream.Length AndAlso ReadChunk(Reader) <> "data"

        End While

        If m_Stream.Position >= m_Stream.Length Then
            Throw New Exception("Invalid file format")
        End If

        m_Length = Reader.ReadInt32()
        m_DataPos = m_Stream.Position

        Position = 0
    End Sub

    Public Sub New(ByVal fileName As String)
        Me.New(New FileStream(fileName, FileMode.Open, FileAccess.Read))
    End Sub

    Public Sub New(ByVal S As Stream)
        m_Stream = S
        ReadHeader()
    End Sub

    Protected Overrides Sub Finalize()
        Try
            Dispose()
        Finally
            MyBase.Finalize()
        End Try
    End Sub

    'Public Sub Dispose() Implements IDisposable.Dispose
    '    If m_Stream IsNot Nothing Then
    '        m_Stream.Close()
    '    End If
    '    GC.SuppressFinalize(Me)
    'End Sub
    Public Overloads Sub Dispose()
        If m_Stream IsNot Nothing Then
            m_Stream.Close()
        End If
        GC.SuppressFinalize(Me)
    End Sub

    Public Overloads Overrides ReadOnly Property CanRead() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overloads Overrides ReadOnly Property CanSeek() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overloads Overrides ReadOnly Property CanWrite() As Boolean
        Get
            Return False
        End Get
    End Property

    Public Overloads Overrides ReadOnly Property Length() As Long
        Get
            Return m_Length
        End Get
    End Property

    Public Overloads Overrides Property Position() As Long
        Get
            Return m_Stream.Position - m_DataPos
        End Get
        Set(ByVal value As Long)
            Seek(value, SeekOrigin.Begin)
        End Set
    End Property

    Public Overloads Overrides Sub Close()
        Dispose()
    End Sub

    Public Overloads Overrides Sub Flush()
    End Sub

    Public Overloads Overrides Sub SetLength(ByVal len As Long)
        Throw New InvalidOperationException()
    End Sub

    Public Overloads Overrides Function Seek(ByVal pos As Long, ByVal o As SeekOrigin) As Long
        Select Case o
            Case SeekOrigin.Begin
                m_Stream.Position = pos + m_DataPos
                Exit Select
            Case SeekOrigin.Current
                m_Stream.Seek(pos, SeekOrigin.Current)
                Exit Select
            Case SeekOrigin.End
                m_Stream.Position = m_DataPos + m_Length - pos
                Exit Select
        End Select
        Return Me.Position
    End Function

    Public Overloads Overrides Function Read(ByVal buf As Byte(), ByVal ofs As Integer, ByVal count As Integer) As Integer
        Dim toread As Integer = CInt(Math.Min(count, m_Length - Position))
        Return m_Stream.Read(buf, ofs, toread)
    End Function

    Public Overloads Overrides Sub Write(ByVal buf As Byte(), ByVal ofs As Integer, ByVal count As Integer)
        Throw New InvalidOperationException()
    End Sub

End Class

